@@ -26,6 +26,7 @@
 
 
 using System;
+using System.IO;
 using System.Collections;
 using System.Reflection;
 using System.Text;
@@ -36,6 +37,8 @@ namespace Beagle.Daemon {
 	
 	public class QueryDriver {
 
+		// Enable or Disable specific Queryables by name
+
 		static ArrayList allowed_queryables = new ArrayList ();
 		static ArrayList denied_queryables = new ArrayList ();
 		
@@ -51,25 +54,33 @@ namespace Beagle.Daemon {
 
 		static private bool UseQueryable (string name)
 		{
+			if (allowed_queryables.Count == 0
+			    && denied_queryables.Count == 0)
+				return true;
+
 			name = name.ToLower ();
 
 			if (allowed_queryables.Count > 0) {
-				foreach (string allowed in allowed_queryables) {
-					if (name == allowed) 
+				foreach (string allowed in allowed_queryables)
+					if (name == allowed)
 						return true;
-				}
 				return false;
 			}
 
-			foreach (string denied in denied_queryables) {
+			foreach (string denied in denied_queryables)
 				if (name == denied)
 					return false;
-			}
 			return true;
 
 		}
 
+		//////////////////////////////////////////////////////////////////////////////////////
+
+		// Use introspection to find all classes that implement IQueryable, the construct
+		// associated Queryables objects.
+
 		static ArrayList queryables = new ArrayList ();
+		static Hashtable iqueryable_to_queryable = new Hashtable ();
 
 		static bool ThisApiSoVeryIsBroken (Type m, object criteria)
 		{
@@ -89,7 +100,8 @@ namespace Beagle.Daemon {
 		// assemble a Queryable object and stick it into our list of queryables.
 		static void ScanAssembly (Assembly assembly)
 		{
-			DateTime t1 = DateTime.Now;
+			int count = 0;
+
 			foreach (Type type in assembly.GetTypes ()) {
 				if (TypeImplementsInterface (type, typeof (IQueryable))) {
 					foreach (object obj in Attribute.GetCustomAttributes (type)) {
@@ -99,121 +111,131 @@ namespace Beagle.Daemon {
 						
 						if (! UseQueryable (flavor.Name))
 							continue;
-						
+
 						IQueryable iq = null;
 						try {
 							iq = Activator.CreateInstance (type) as IQueryable;
 						} catch (Exception e) {
-							Logger.Log.Error ("Caught exception while activiting {0} backend", flavor.Name);
+							Logger.Log.Error ("Caught exception while instantiating {0} backend", flavor.Name);
 							Logger.Log.Error (e);
 						}
-						if (iq == null)
-							continue;
 
-						Queryable q = new Queryable (flavor, iq);
-						queryables.Add (q);
-						break;
+						if (iq != null) {
+							Queryable q = new Queryable (flavor, iq);
+							queryables.Add (q);
+							iqueryable_to_queryable [iq] = q;
+							++count;
+							break;
+						}
 					}
 				}
 			}
-			DateTime t2 = DateTime.Now;
-
-			Logger.Log.Debug ("Scanned assembly '{0}' in {1:0.00}s", assembly, (t2-t1).TotalSeconds);
+			Logger.Log.Debug ("Found {0} types in {1}", count, assembly.FullName);
 		}
 
-		static bool initialized = false;
-
-		static void Initialize ()
+		static public void Start ()
 		{
-			lock (queryables) {
-				if (initialized)
-					return;
-				ScanAssembly (Assembly.GetCallingAssembly ());
-				initialized = true;
+			ScanAssembly (Assembly.GetExecutingAssembly ());
+			
+			DirectoryInfo backends = new DirectoryInfo (PathFinder.BackendDir);
+
+			if (backends.Exists) {
+				foreach (FileInfo assembly in backends.GetFiles ("*.dll"))
+					ScanAssembly (Assembly.LoadFile (assembly.ToString ()));
 			}
+				
+			foreach (Queryable q in queryables)
+				q.Start ();
 		}
 
 		////////////////////////////////////////////////////////
 
-		public delegate void ChangedHandler (QueryDriver          source,
-						     Queryable            queryable,
+		public delegate void ChangedHandler (Queryable            queryable,
 						     IQueryableChangeData changeData);
 
-		public event ChangedHandler ChangedEvent;
+		static public event ChangedHandler ChangedEvent;
 
-
-		// FIXME: Would we have problems if there were multiple QueryDriver
-		// objects floating around?
-		
-		// FIXME: There should be a way to disconnect this OnQueryableChanged
-		// from the ChangedEvents.
-		public QueryDriver ()
-		{
-			Initialize ();
-			foreach (Queryable q in queryables) {
-				q.ChangedEvent += OnQueryableChanged;
-			}
-		}
-
-		public void Start ()
-		{
-			foreach (Queryable q in queryables) {
-				q.Start ();
-			}
-		}
-
-		public void DoQuery (QueryBody body, QueryResult result)
-		{
-			// The extra pair of calls to WorkerStart/WorkerFinished ensures that
-			// the QueryResult will fire the StartedEvent and FinishedEvent,
-			// even if no queryable accepts the query.
-			if (!result.WorkerStart (this)) {
-				return;
-			}
-			
-			try {
-				foreach (Queryable queryable in queryables) {
-					if (queryable.AcceptQuery (body))
-						queryable.DoQuery (body, result, null);
-				}
-			} finally {
-				result.WorkerFinished (this);
-			}
-		}
-
-		public void DoQueryChange (Queryable            queryable,
-					   IQueryableChangeData changeData,
-					   QueryBody            body,
-					   QueryResult          result)
-		{
-			if (queryable.AcceptQuery (body)) 
-				queryable.DoQuery (body, result, changeData);
-		}
-
-		private void OnQueryableChanged (Queryable            source,
-						 IQueryableChangeData changeData)
+		// A method to fire the ChangedEvent event.
+		static public void QueryableChanged (IQueryable           iqueryable,
+						     IQueryableChangeData change_data)
 		{
 			if (ChangedEvent != null) {
-				ChangedEvent (this, source, changeData);
+				Queryable queryable = iqueryable_to_queryable [iqueryable] as Queryable;
+				ChangedEvent (queryable, change_data);
 			}
 		}
 
 		////////////////////////////////////////////////////////
 
-		public string GetHumanReadableStatus ()
+		private class QueryClosure : IQueryWorker {
+
+			Queryable queryable;
+			QueryBody body;
+			QueryResult result;
+			IQueryableChangeData change_data;
+			
+			public QueryClosure (Queryable            queryable,
+					     QueryBody            body,
+					     QueryResult          result,
+					     IQueryableChangeData change_data)
+			{
+				this.queryable = queryable;
+				this.body = body;
+				this.result = result;
+				this.change_data = change_data;
+			}
+
+			public void DoWork ()
+			{
+				HitRegulator regulator = result.GetHitRegulator (queryable);
+				queryable.DoQuery (body, regulator, change_data);
+				regulator.Flush (result);
+			}
+		}
+
+		static public void DoOneQuery (Queryable            queryable,
+					       QueryBody            body,
+					       QueryResult          result,
+					       IQueryableChangeData change_data)
+		{
+			if (queryable.AcceptQuery (body)) {
+				QueryClosure qc = new QueryClosure (queryable, body, result, change_data);
+				result.AttachWorker (qc);
+			}
+		}
+
+		static public void DoQuery (QueryBody body, QueryResult result)
+		{
+			// The extra pair of calls to WorkerStart/WorkerFinished ensures:
+			// (1) that the QueryResult will fire the StartedEvent
+			// and FinishedEvent, even if no queryable accepts the
+			// query.
+			// (2) that the FinishedEvent will only get called when all of the
+			// backends have had time to finish.
+
+			object dummy_worker = new object ();
+
+			if (! result.WorkerStart (dummy_worker))
+				return;
+			
+			foreach (Queryable queryable in queryables)
+				DoOneQuery (queryable, body, result, null);
+			
+			result.WorkerFinished (dummy_worker);
+		}
+
+		////////////////////////////////////////////////////////
+
+		static public string GetIndexInformation ()
 		{
 			StringBuilder builder = new StringBuilder ("\n");
 
 			foreach (Queryable q in queryables) {
-				if (builder.Length > 1)
-					builder.Append ("\n--------------------------------------\n\n");
-				builder.Append (q.Name);
-				builder.Append (":\n");
-				builder.Append (q.GetHumanReadableStatus ());
+				builder.AppendFormat ("Name: {0}\n", q.Name);
+				builder.AppendFormat ("Count: {0}\n", q.GetItemCount ());
 				builder.Append ("\n");
 			}
-			builder.Append ("\n");
-			
+
 			return builder.ToString ();
 		}
 	}
